@@ -269,13 +269,22 @@ duo_call(struct duo_ctx *ctx, const char *method, const char *uri)
                 _duo_seterr(ctx, "Couldn't connect to %s: %s\n",
                     ctx->host, https_geterr());
         } else if (code / 100 == 2) {
+                /* 2xx indicates DUO_OK */
                 ret = DUO_OK;
+        } else if (code == 401) {
+                /* 401 indicates an invalid ikey or skey */
+                ret = DUO_CLIENT_ERROR;
+                _duo_seterr(ctx, "Invalid ikey or skey");
+        } else if (code / 100 == 5) {
+                /* 5xx indicates an internal server error */
+                ret = DUO_SERVER_ERROR;
+                _duo_seterr(ctx, "HTTP %d", code);
         } else {
-                ret = (code < 500) ? DUO_CLIENT_ERROR : DUO_SERVER_ERROR;
-                if (_duo_bson_response(ctx, NULL) != DUO_FAIL)
-                        _duo_seterr(ctx, "HTTP %d", code);
+                /* abort on any other HTTP codes */
+                ret = DUO_ABORT;
+                _duo_seterr(ctx, "HTTP %d", code);
         }
-	return (ret);
+        return (ret);
 }
 
 const char *
@@ -400,6 +409,7 @@ duo_login(struct duo_ctx *ctx, const char *username,
 	bson_iterator it;
 	duo_code_t ret;
 	char buf[256];
+	char *pushinfo = NULL;
 	const char *p;
 	int i;
 
@@ -407,38 +417,42 @@ duo_login(struct duo_ctx *ctx, const char *username,
 		_duo_seterr(ctx, "need username to authenticate");
 		return (DUO_CLIENT_ERROR);
 	}
-        /* Check preauth status */
-        if ((ret = _duo_preauth(ctx, &obj, username)) != -1) {
-                return (ret);
-        }
+
+	/* Check preauth status */
+	if ((ret = _duo_preauth(ctx, &obj, username)) != -1) {
+		return (ret);
+	}
+
 	/* Handle factor selection */
-        if ((ret = _duo_prompt(ctx, &obj, flags, buf, sizeof(buf), &p)) != -1) {
-                return (ret);
-        }
-	/* Try Duo authentication */
+	if ((ret = _duo_prompt(ctx, &obj, flags, buf, sizeof(buf), &p)) != -1) {
+		return (ret);
+	}
+
+	/* Add request parameters */
 	if (duo_add_param(ctx, "user", username) != DUO_OK ||
 	    duo_add_param(ctx, "factor", "auto") != DUO_OK ||
 	    duo_add_param(ctx, "auto", p) != DUO_OK ||
 	    duo_add_param(ctx, "async",
-		(flags & DUO_FLAG_SYNC) ? "0" : "1") != DUO_OK ||
+	    (flags & DUO_FLAG_SYNC) ? "0" : "1") != DUO_OK ||
 	    duo_add_param(ctx, "ipaddr",
-		client_ip ? client_ip : _local_ip()) != DUO_OK) {
+	    client_ip ? client_ip : _local_ip()) != DUO_OK) {
 		return (DUO_LIB_ERROR);
 	}
-        if (command != NULL) {
-                char *p, *v;
-                if ((v = urlenc_encode(command)) == NULL || 
-                    asprintf(&p, "Command=%s", v) < 0) {
-                        free(v);
-                        return (DUO_LIB_ERROR);
-                }
-                duo_add_param(ctx, "pushinfo", p);
-                free(p);
-        }
+
+	/* Add pushinfo parameters */
+	if (asprintf(&pushinfo, "Server+IP=%s&Command=%s",
+	    _local_ip(), command ? urlenc_encode(command) : "") < 0 ||
+	    duo_add_param(ctx, "pushinfo", pushinfo) != DUO_OK) {
+		return (DUO_LIB_ERROR);
+	}
+	free(pushinfo);
+
+	/* Try Duo authentication */
 	if ((ret = duo_call(ctx, "POST", DUO_API_VERSION "/auth.bson")) != DUO_OK ||
 	    (ret = _duo_bson_response(ctx, &obj)) != DUO_OK) {
 		return (ret);
 	}
+
 	/* Handle sync status */
 	if ((flags & DUO_FLAG_SYNC) != 0) {
 		_BSON_FIND(ctx, &it, &obj, "status", bson_string);
